
(define-macro (while test . body)
  `(do () ((not ,test))
    ,@body
  )
)

(define (current-module)
  (interaction-environment)
)


(define-macro (assoc-set! alist id new)
  `(let ((nlist '()))
    (if (assoc ,id ,alist)
      ; in the assoc list
      (begin
        (for-each
          (lambda (x)
            (if (equal? (car x) ,id)
              (set! nlist (cons (cons (car x) ,new) nlist))
              (set! nlist (cons x nlist))
            )
          )
          ,alist
        )
        (reverse! nlist)
      )
      ; new element
      (cons (cons ,id ,new) ,alist)
    )
  )
)

(define-macro (assoc-remove! alist id)
  `(let ((nlist '()))
    (for-each
      (lambda (x)
        (unless (equal? (car x) ,id)
          (set! nlist (cons x nlist))
        )
      )
      ,alist
    )
    (reverse! nlist)
  )
)


(define-macro (assoc-ref alist id)
  `(let ((res (assoc ,id ,alist)))
    (if res
      (cdr res)
      #f
    )
  )
)

(define-macro (1+ number)
  `(+ ,number 1)
)

(define-macro (1- number)
  `(- ,number 1)
)

; --------------------------------------

(define file-line-number 0)
(define unread-buffer '())
(define my-peek peek-char)
(define my-read read-char)

(define (peek-char port)
  (if (null? unread-buffer)
    (my-peek port)
    (car unread-buffer)
  )
)

(define (read-char port)
  (if (null? unread-buffer)
    (let* ( (c (my-read port)) )
      (if (char=? c #\newline)
        (set! file-line-number (+ 1 file-line-number))
      )
      c
    )
    (let* ( (c (car unread-buffer)) )
      (set! unread-buffer (cdr unread-buffer))
      (if (char=? c #\newline)
        (set! file-line-number (+ 1 file-line-number))
      )
      c
    )
  )
)

(define (unread-string str port)
  (set! unread-buffer (string->list str))
)

(define (port-column port)
  -1
)

(define (port-line port)
  file-line-number
)


(define (make-array def row col)
  (let* ( (m (make-vector row def)) )
    (do ((i 0 (+ 1 i))) ((= i row))
      (vector-set! m i (make-vector col def))
    )
    m
  )
)

(define (array-set! arr val row col)
  (vector-set! (vector-ref arr row) col val)
)
